from django.conf import settings
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from .managers import ActiveManager
from .settings import SCORE_CHOICES


@python_2_unicode_compatible
class Review(models.Model):
    """
    A ``Review`` consists on a comment and a rating.
    """
    content_type = models.ForeignKey(
        ContentType, verbose_name=_("Content type"), related_name="content_type_set_for_%(class)s",
        null=True, blank=True)
    object_id = models.PositiveIntegerField(_("Content ID"), blank=True, null=True)
    content = GenericForeignKey(ct_field="content_type", fk_field="object_id")

    # if the user is authenticated we save the user otherwise the name and the email.
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, verbose_name=_("User"), blank=True, null=True,
        related_name="%(class)s_comments", on_delete=models.SET_NULL)
    user_name = models.CharField(_("Name"), max_length=50, blank=True)
    user_email = models.EmailField(_("E-mail"), blank=True)

    active = models.BooleanField(_("Active"), default=False)
    title = models.CharField(_("Title"), default='', max_length=300)
    comment = models.TextField(_("Comment"), blank=True)
    score = models.PositiveIntegerField(_("Score"), choices=SCORE_CHOICES)
    display_date = models.DateTimeField(_("Display date"), null=True)
    creation_date = models.DateTimeField(_("Creation date"), auto_now_add=True)
    highlight = models.BooleanField(_("highlight"), default=False, blank=True)
    source = models.TextField(_("Source"), default='', blank=True)

    objects = ActiveManager()

    class Meta:
        ordering = ('-display_date', )

    def __str__(self):
        return "{} ({})".format(self.name, self.score)

    @property
    def name(self):
        """
        Returns the stored user name.
        """
        if self.user is not None:
            return self.user.get_full_name()
        else:
            return self.user_name

    @property
    def email(self):
        """
        Returns the stored user email.
        """
        if self.user is not None:
            return self.user.email
        else:
            return self.user_email



from django import forms
from django.utils.translation import ugettext_lazy as _

from .models import Review


class ReviewAdminForm(forms.ModelForm):
    class Meta:
        model = Review
        fields = (
            'active',
            'highlight',
            'score',
            'title',
            'comment',
            'display_date',
            'user',
            'user_name',
            'user_email',
            'source',
            'object_id',
            'content_type',
        )

    def clean(self):
        cleaned_data = self.cleaned_data
        highlight = cleaned_data.get('highlight', False)

        if highlight:
            queryset = Review.objects.filter(active=True, highlight=True, content_type=self.instance.content_type, object_id=self.instance.object_id)
            if self.instance.pk:
                queryset = queryset.exclude(pk=self.instance.pk)

            if queryset.exists():
                self.add_error('highlight', _('There already is a highlighted review for this object'))

        return cleaned_data



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-14 12:17
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('reviews', '0003_auto_20170912_1622'),
    ]

    operations = [
        migrations.AlterField(
            model_name='review',
            name='content_type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='content_type_set_for_review', to='contenttypes.ContentType', verbose_name='Content type'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-12 09:54
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('reviews', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='review',
            name='score',
            field=models.PositiveIntegerField(choices=[(1, '*'), (2, '**'), (3, '***'), (4, '****'), (5, '*****')], verbose_name='Score'),
        ),
    ]



from django.db.models import Manager, Avg
from django.db.models.query import QuerySet

from django.contrib.contenttypes.models import ContentType


class ActiveMixin(object):
    """
    An extended manager to return active objects.
    """
    def active(self):
        return self.filter(active=True)

    def average(self, instance=None):
        queryset = self
        if instance:
            queryset = queryset.filter(object_id=instance.id, content_type=ContentType.objects.get_for_model(instance))
        avg_dict = queryset.aggregate(average=Avg('score'))
        return avg_dict.get('average', 0)


class ActiveQuerySet(ActiveMixin, QuerySet):
    pass


class ActiveManager(ActiveMixin, Manager):
    def get_queryset(self):
        return ActiveQuerySet(self.model, using=self._db)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-09-15 09:48
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('reviews', '0004_auto_20170914_1417'),
    ]

    operations = [
        migrations.AlterField(
            model_name='review',
            name='score',
            field=models.PositiveIntegerField(choices=[(1, '1'), (2, '2'), (3, '3'), (4, '4'), (5, '5'), (6, '6'), (7, '7'), (8, '8'), (9, '9'), (10, '10')], verbose_name='Score'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2018-06-18 08:39
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        ('reviews', '0011_auto_20180611_1422'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='review',
            unique_together=set([('highlight', 'content_type', 'object_id')]),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2018-06-11 09:45
from __future__ import unicode_literals

from django.db import migrations


def set_display_date(apps, schemas):
    Review = apps.get_model('reviews', 'Review')
    for review in Review.objects.all():
        review.display_date = review.creation_date
        review.save()


class Migration(migrations.Migration):

    dependencies = [
        ('reviews', '0008_review_highlight'),
    ]

    operations = [
        migrations.RunPython(set_display_date)
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2018-06-11 09:33
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('reviews', '0005_auto_20170915_1148'),
    ]

    operations = [
        migrations.AddField(
            model_name='review',
            name='title',
            field=models.CharField(default='', max_length=300, verbose_name='Title'),
        ),
    ]
